/* examples for the jcifs smb client library in Java
 * Copyright (C) 2001  "Michael B. Allen" <mballen@erols.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import jcifs.smb.*;
import java.net.MalformedURLException;

public class SmbShell {

    public static String readLine() throws Exception {
        int c;
        StringBuffer sb = new StringBuffer();
        while(( c = System.in.read() ) != '\n' ) {
            sb.append( (char)c );
        }
        return sb.toString().trim();
    }

    public static void main( String[] argv ) throws Exception {
        int c;
        String cmd, prompt;
		SmbFile conn = null, tmp;
        while( true ) {
			try {
				prompt = "> ";
				if( conn != null ) {
					if( conn.exists() ) {
						prompt = conn.getCanonicalPath();
						prompt = prompt.substring(
									prompt.lastIndexOf( '/' ) + 1,
									prompt.length() ) + "> ";
					} else {
						System.out.println( "no such directory" );
					}
				}
	            System.out.print( prompt );
	
	            cmd = readLine();
	            if( cmd.equals( "" ) ) {
	            } else if( cmd.startsWith( "ope" )) {
	                int i = cmd.indexOf( ' ' );
	                String url;
	                if( i == -1 || (url = cmd.substring( i ).trim()).length() == 0 ) {
	                    System.out.println(
								"usage: open smb://[[domain;]user[:pass]@]host/share[/dir]" );
	                    continue;
	                }
					conn = new SmbFile( url );
	            } else if( cmd.startsWith( "cd" )) {
	                int i = cmd.indexOf( ' ' );
	                String dir;
	                if( i == -1 || (dir = cmd.substring( i ).trim()).length() == 0 ) {
	                    System.out.println( "usage: cd dir" );
	                    continue;
	                }
					tmp = new SmbFile( conn.getCanonicalPath(), dir );
					if( tmp.exists() ) {
						conn = tmp;
					} else {
						System.out.println( "no such directory" );
					}
	            } else if( cmd.startsWith( "ls" )) {
	                int i = cmd.indexOf( ' ' );
					SmbFile d = conn;
	                String dir;
	                if( i != -1 && (dir = cmd.substring( i ).trim()).length() != 0 ) {
						d = new SmbFile( conn, dir );
	                }
					String[] list = d.list();
					if( list != null ) {
						for( int j = 0; j < list.length; j++ ) {
							System.out.print( " " + list[j] );
						}
						System.out.println();
						System.out.println( list.length + " items" );
					} else {
						System.out.println( "no such file or directory" );
					}
	            } else if( cmd.startsWith( "pwd" )) {
					System.out.println( conn.getCanonicalPath() );
	            } else if( cmd.startsWith( "q" ) ||
	                        cmd.startsWith( "x" ) ||
	                        cmd.startsWith( "ex" ) ||
	                        cmd.startsWith( "by" )) {
	                break;
	            } else {
	                System.out.println( "commands:" );
	                System.out.println( "  open smb://[[domain;]user[:pass]@]host/share[/dir]" );
	                System.out.println( "  ls [dir|file]" );
	                System.out.println( "  cd dir" );
	                System.out.println( "  pwd" );
	                System.out.println( "  quit" );
	            }
			} catch( MalformedURLException mue ) {
				mue.printStackTrace();
				conn = null;
			}
        }
		System.exit( 0 );
    }
}
