/* examples for the jcifs smb client library in Java
 * Copyright (C) 2001  "Michael B. Allen" <mballen@erols.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import jcifs.smb.SmbFile;
import jcifs.smb.SmbFileInputStream;
import jcifs.smb.SmbFileOutputStream;
import java.io.IOException;
import java.util.Date;

public class TortureTest1 {

	static final byte[] data = new String( "hello, world\r\n" ).getBytes();

	public static void main( String[] argv ) throws Exception {

		if( argv.length != 1 ) {
			throw new IllegalArgumentException( "must provide smb url" );
		}

		SmbFile f0 = new SmbFile( argv[0] );
		if( f0.exists() ) {
			throw new IllegalArgumentException( "file already exists" );
		}
		System.out.print( "writing file: " + f0.getCanonicalPath() + " ... " );
		SmbFileOutputStream out = new SmbFileOutputStream( f0 );
		out.write( data );
		out.close();
		if( f0.exists() == false ) {
			throw new IOException( "failed to write file: " + f0.getCanonicalPath() );
		} else {
			System.out.println( "succeess" );
		}
		System.out.println( "getCanonicalPath: " + f0.getCanonicalPath() );
		System.out.println( "         getName: " + f0.getName() );
		System.out.println( "       getParent: " + f0.getParent() );
		System.out.println( "         getPath: " + f0.getPath() );
		System.out.println( "        toString: " + f0.toString() );

		System.out.println( "          exists: " + f0.exists() );
		System.out.println( "         canRead: " + f0.canRead() );
		System.out.println( "        canWrite: " + f0.canWrite() );
		System.out.println( "     isDirectory: " + f0.isDirectory() );
		System.out.println( "          isFile: " + f0.isFile() );
		System.out.println( "        isHidden: " + f0.isHidden() );
		System.out.println( "    lastModified: " + new Date( f0.lastModified() ));

		SmbFile f1 = new SmbFile( f0.getParent() + "/renamed.tmp" );
		System.out.print( "renaming: " + f0.getCanonicalPath() + " to: " + f1.getCanonicalPath() + " ... " );
		if( f0.renameTo( f1 ) == false ) {
			throw new IOException( "failed to rename file" );
		}
		System.out.println( "succeess" );

		if( f1.exists() == false ) {
			throw new IOException( "renaming suceeded but file still doesn't exist?" );
		}
		if( f0.exists() ) {
			throw new IOException( "renaming suceeded but old file still exists?" );
		}
		System.out.println( "listing parent directory ... " );
		SmbFile dir = new SmbFile( f0, ".." );
		if( dir.exists() == false ) {
			throw new IOException( "parent directory does not exist: " + dir.getCanonicalPath() );
		}
		String[] files = dir.list();
		for( int i = 0; i < files.length; i++ ) {
			System.out.print( " " + files[i] );
		} 
		System.out.println();
		System.out.println( "success" );

		for( int i = 0; i < files.length; i++ ) {
			System.out.println( files[i] + "    " + (new SmbFile( dir, files[i] )).length() );
		} 
	}
}
