/* examples for the jcifs smb client library in Java
 * Copyright (C) 2001  "Michael B. Allen" <mballen@erols.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import jcifs.smb.SmbFile;
import jcifs.smb.SmbFileInputStream;
import jcifs.smb.SmbFileOutputStream;
import java.io.IOException;
import java.io.FileInputStream;
import java.util.Date;

public class TortureTest2 {

	public static void main( String[] argv ) throws Exception {

		if( argv.length != 1 ) {
			throw new IllegalArgumentException( "must provide smb url" );
		}

		SmbFile file = new SmbFile( argv[0] );
		SmbFileOutputStream out = new SmbFileOutputStream( file );
		FileInputStream in = new FileInputStream( file.getName() );

		// put file
		byte[] b = new byte[4096];
		int n;
		while(( n = in.read( b )) > 0 ) {
			out.write( b, 0, n );
		}
		in.close();
		out.close();
		System.out.println( "wrote: " + file );

		SmbFile dir = new SmbFile( file.getParent() + "/dir0" );
		if( dir.mkdir() ) {
			System.out.println( "created directory: " + dir );
		} else {
			System.out.println( "failed to create directory: " + dir );
		}

		SmbFile file2 = new SmbFile( file, "../dir0/file2.ren" );
		if( file.renameTo( file2 )) {
			System.out.println( "renamed: " + file + " to: " + file2 );
		} else {
			System.out.println( "failed to rename: " + file + " to: " + file2 );
		}

		SmbFile dir2 = new SmbFile( file, "../dir2" );
		if( dir.renameTo( dir2 )) {
			System.out.println( "renamed: " + dir + " to: " + dir2 );
		} else {
			System.out.println( "failed to rename: " + dir + " to: " + dir2 );
		}

/** Cannot delete a directory if it has anything in it. Will need recursive func(post order DFS). **/

		SmbFile file3 = new SmbFile( dir2, "file2.ren" );
		if( file3.delete() ) {
			System.out.println( "deleted: " + file3 );
		} else {
			System.out.println( "failed to delete: " + file3 );
		}

		if( dir2.delete() ) {
			System.out.println( "deleted: " + dir2 );
		} else {
			System.out.println( "failed to delete: " + dir2 );
		}
	}
}

