/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.netbios;

import java.io.IOException;
import java.io.InputStream;

class NegativeSessionResponsePacket extends SessionServicePacket {

	int errorCode;
	String errorString;

	NegativeSessionResponsePacket() {
		type = NEGATIVE_SESSION_RESPONSE;
		length = 1;
	}
	NegativeSessionResponsePacket( int errorCode ) {
		this.errorCode = errorCode;
	}

	int writeTrailerWireFormat( byte[] dst, int dstIndex ) {
		dst[dstIndex] = (byte)errorCode;
		return 1;
	}
	int readTrailerWireFormat( InputStream in,
							byte[] buffer,
							int bufferIndex )
							throws IOException {
		errorCode = in.read();
		if( errorCode == -1 ) {
			return -1;
		}
		buffer[bufferIndex] = (byte)errorCode;
		switch( errorCode ) {
			case 0x80:
				errorString = "Not listening on called name";
				break;
			case 0x81:
				errorString = "Not listening for calling name";
				break;
			case 0x82:
				errorString = "Called name not present";
				break;
			case 0x83:
				errorString = "Called name present, but insufficient resources";
				break;
			case 0x8F:
				errorString = "Unspecified error";
				break;
			default:
				errorString = "Unknown netbios negative session response error: " +
							"0x" + Name.toHexChars( errorCode );
		}
		return 1;
	}
}
