/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.netbios;

class NodeStatusResponse extends NameServicePacket {

	NbtAddress queryAddress;

	int numberOfNames;
	NbtAddress[] addressArray;
	byte[] macAddress;
	byte[] stats;

	/* It is a little awkward but prudent to pass the quering address
	 * so that it may be included in the list of results. IOW we do
	 * not want to create a new NbtAddress object for this particular
	 * address from which the query is constructed, we want to populate
     * the data of the existing address that should be one of several
	 * returned by the node status.
	 */

	NodeStatusResponse( NbtAddress queryAddress ) {
		this.queryAddress = queryAddress;
		recordName = new Name( null, 0, null );
        macAddress = new byte[6];
	}

	int writeBodyWireFormat( byte[] dst, int dstIndex ) {
		return 0;
	}
	int readBodyWireFormat( byte[] src, int srcIndex ) {
		return readResourceRecordWireFormat( src, srcIndex );
	}
	int writeRDataWireFormat( byte[] dst, int dstIndex ) {
		return 0;
	}
	int readRDataWireFormat( byte[] src, int srcIndex ) {
		int start = srcIndex;
		numberOfNames = src[srcIndex] & 0xFF;
		int namesLength = numberOfNames * 18;
		int statsLength = rDataLength - namesLength - 1;
		numberOfNames = src[srcIndex++] & 0xFF;
		// gotta read the mac first so we can populate addressArray with it
        System.arraycopy( src, srcIndex + namesLength, macAddress, 0, 6 );
		srcIndex += readNodeNameArray( src, srcIndex );
		stats = new byte[statsLength];
		System.arraycopy( src, srcIndex, stats, 0, statsLength );
		srcIndex += statsLength;
		return srcIndex - start;
	}
    private int readNodeNameArray( byte[] src, int srcIndex ) {
		int start = srcIndex;

		addressArray = new NbtAddress[numberOfNames];

        String n;
        int type;
        String scope = queryAddress.hostName.scope;
        boolean groupName;
        int ownerNodeType;
        boolean isBeingDeleted;
        boolean isInConflict;
        boolean isActive;
        boolean isPermanent;
        int j;
        boolean addrFound = false;
        for( int i = 0; i < numberOfNames; srcIndex += 18, i++ ) {
            for( j = srcIndex + 14; src[j] == 0x20; j-- )
                ;
            n = new String( src, srcIndex, j - srcIndex + 1 );
            type             =    src[srcIndex + 15] & 0xFF;
            groupName        = (( src[srcIndex + 16] & 0x80 ) == 0x80 ) ? true : false;
            ownerNodeType    =  ( src[srcIndex + 16] & 0x60 ) >> 5;
            isBeingDeleted   = (( src[srcIndex + 16] & 0x10 ) == 0x10 ) ? true : false;
            isInConflict     = (( src[srcIndex + 16] & 0x08 ) == 0x08 ) ? true : false;
            isActive         = (( src[srcIndex + 16] & 0x04 ) == 0x04 ) ? true : false;
            isPermanent      = (( src[srcIndex + 16] & 0x02 ) == 0x02 ) ? true : false;
            if( !addrFound && queryAddress.hostName.type == type &&
                    ( queryAddress.hostName.name == null ||
					queryAddress.hostName.name.equals( NbtAddress.ALL_HOSTS_NAME ) ||
					queryAddress.hostName.name.equalsIgnoreCase( n ))) {
                queryAddress.hostName.name = n;
                queryAddress.groupName = groupName;
                queryAddress.nodeType = ownerNodeType;
                queryAddress.isBeingDeleted = isBeingDeleted;
                queryAddress.isInConflict = isInConflict;
                queryAddress.isActive = isActive;
                queryAddress.isPermanent = isPermanent;
                queryAddress.macAddress = macAddress;
                queryAddress.isDataFromNodeStatus = true;
                addrFound = true;
                addressArray[i] = queryAddress;
            } else {
                addressArray[i] = new NbtAddress( new Name( n, type, scope ),
                                        queryAddress.address,
                                        groupName,
                                        ownerNodeType,
                                        isBeingDeleted,
                                        isInConflict,
                                        isActive,
                                        isPermanent,
                                        macAddress );
            }
        }
        return srcIndex - start;
    }
}

