/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.netbios;

import java.io.FilterOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import jcifs.util.Config;
import jcifs.util.Log;

class SocketOutputStream extends FilterOutputStream {

	private static int DEFAULT_BUFR_SIZE = 1500;

	private byte[] bufr;
	private int count, writeSize;

	SocketOutputStream( OutputStream out ) {
		this( out, DEFAULT_BUFR_SIZE );
	}
	SocketOutputStream( OutputStream out, int size ) {
		super( out );
		writeSize = Config.getInt( "jcifs.netbios.client.writeSize", size );
		if( writeSize <= 0 ) {
			throw new IllegalArgumentException( "buffer size <= 0" );
		} else if( writeSize > SessionServicePacket.MAX_MESSAGE_SIZE ) {
			throw new IllegalArgumentException( "buffer exceeds max netbios message size" );
		}
		writeSize += 4;
		bufr = new byte[writeSize];
		bufr[0] = (byte)SessionServicePacket.SESSION_MESSAGE;
		count = 4;
	}

	public synchronized void write( int b ) throws IOException {
		if( count >= bufr.length ) {
			flush();
		}
		bufr[count++] = (byte)b;
	}
	public synchronized void write( byte[] b, int off, int len ) throws IOException {
		if( len >= bufr.length ) {
			Log.println( Log.WARNINGS, "session service warning",
							"write len exceeds pre-allocated buffer size; performance " +
							"will suffer: len=" + len + ",writeSize=" + writeSize );
			flush();
			byte[] tmp = bufr;
			bufr = new byte[len + 4]; /* at least mantain a permainent fixed size buffer */
			System.arraycopy( b, off, bufr, count, len ); /* doah! */
			count += len;
			flush();
			bufr = tmp;
			return;
		}
		if( len > bufr.length - count ) {
			flush();
		}
		System.arraycopy( b, off, bufr, count, len );
		count += len;
	}
	public synchronized void flush() throws IOException {
		if( count == 4 ) {
			return;
		}
		int n = count - 4;
		bufr[1] = (byte)(( n & 0x010000 ) == 0x00 ? 0x00 : 0x01 ); 
		bufr[2] = (byte)(( n >> 8 ) & 0xFF ); 
		bufr[3] = (byte)( n & 0xFF ); 
		out.write( bufr, 0, count );
		out.flush();
		count = 4;
	}
}
