/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

/**
 * Provides logging methods specific to the smb package. Log class
 * members are applied to the Log's mask(set as an arithmetic expression
 * from {@link jcifs.util.Log#setMask(int mask)}) to "flip on" logging of
 * smb specific information.
 * <p><blockquote><pre>
 *     Log.setMask( Log.EXCEPTIONS +
 *                 jcifs.smb.Log.MESSAGE_DATA );
 * </pre></blockquote>
 * <p>
 * See the {@link jcifs.util.Log} parent class for details about this
 * logging style.
 *
 * @version   0.1, 00/03/13
 * @author    Michael B. Allen
 * @see       jcifs.util.Log
 * @since     jcifs-0.1
 */

public class Log extends jcifs.util.Log {

    // supress javadoc constructor summary
    Log() {}

    public static final int MESSAGE_DATA     = 0x00000100;
    public static final int TEST1            = 0x00000200;
    public static final int RESERVED2        = 0x00000400;
    public static final int RESERVED3        = 0x00000800;

	static void printMessageData( String desc, ServerMessageBlock smb ) {
		if(( mask & MESSAGE_DATA ) == 0 ) {
			return;
		}
		try {
            out.println( desc, smb.toString() + NL );
        } catch( Exception e ) {
            Log.printStackTrace( "jcifs.smb.Log.printMessageData()", e );
        }
	}
}

