/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

class SmbComNTCreateAndX extends AndXServerMessageBlock {

	// access mask encoding
	static final int FILE_READ_DATA        = 0x00000001; // 1
	static final int FILE_WRITE_DATA       = 0x00000002; // 2
	static final int FILE_APPEND_DATA      = 0x00000004; // 3
	static final int FILE_READ_EA          = 0x00000008; // 4
	static final int FILE_WRITE_EA         = 0x00000010; // 5
	static final int FILE_EXECUTE          = 0x00000020; // 6
	static final int FILE_DELETE           = 0x00000040; // 7
	static final int FILE_READ_ATTRIBUTES  = 0x00000080; // 8
	static final int FILE_WRITE_ATTRIBUTES = 0x00000100; // 9
	static final int DELETE                = 0x00010000; // 16
	static final int READ_CONTROL          = 0x00020000; // 17
	static final int WRITE_DAC             = 0x00040000; // 18
	static final int WRITE_OWNER           = 0x00080000; // 19
	static final int SYNCHRONIZE           = 0x00100000; // 20
	static final int GENERIC_ALL           = 0x10000000; // 28
	static final int GENERIC_EXECUTE       = 0x20000000; // 29
	static final int GENERIC_WRITE         = 0x40000000; // 30
	static final int GENERIC_READ          = 0x80000000; // 31

	// share access
	static final int FILE_SHARE_READ   = 0x01;
	static final int FILE_SHARE_WRITE  = 0x02;
	static final int FILE_SHARE_DELETE = 0x04;

	// create disposition

	/* I only have 90% confidence in these values.
	 */
	// Creates a new file but fails if it already exists.
	static final int CREATE_NEW        = 0x02;
	// Creates a new file overwriting if it already exists.
	static final int CREATE_ALWAYS     = 0x05;
	// Opens a file but fails if it does not exist.
	static final int OPEN_EXISTING     = 0x01;
	// Opens the file but creates a new file if it does not exist
	static final int OPEN_ALWAYS       = 0x03;
	// Opens and truncates the file but fails if it does not exist
	static final int TRUNCATE_EXISTING = 0x04;

	// create options
	static final int FILE_WRITE_THROUGH           = 0x00000002;
	static final int FILE_SEQUENTIAL_ONLY         = 0x00000004;
	static final int FILE_SYNCHRONOUS_IO_ALERT    = 0x00000010;
	static final int FILE_SYNCHRONOUS_IO_NONALERT = 0x00000020;

	String name;
	int flags,
		rootDirectoryFid,
		desiredAccess,
		extFileAttributes,
		shareAccess,
		createDisposition,
		createOptions,
		impersonationLevel;
	long allocationSize;
	byte securityFlags;

	SmbComNTCreateAndX( String name, ServerMessageBlock andx ) {
		super( andx );
		this.name = name;
		command = SMB_COM_NT_CREATE_ANDX;

		/* This is all wrong for writing. Apparently it's
		 * wrong for reading too as Samba gives ERRDOS/ERRbadfid
		 * on SmbComReadAndX :~(
		 *
		 * Need to figure out these all these pesky bits, many of
		 * which are not documented fully.
		 */

		desiredAccess = FILE_READ_DATA | FILE_READ_EA | FILE_READ_ATTRIBUTES;
		extFileAttributes = ATTR_NORMAL;
		shareAccess = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
		createDisposition = OPEN_EXISTING;
		createOptions = 0x00; // ??
		impersonationLevel = 0x02; // As seen on NT :~)
		securityFlags = (byte)0x00;
	}

	int writeParameterWordsWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		dst[dstIndex++] = (byte)0x00;
		writeInt2( stringWireLength( name ), dst, dstIndex );
		dstIndex += 2;
		writeInt4( flags, dst, dstIndex );
		dstIndex += 4;
		writeInt4( rootDirectoryFid, dst, dstIndex );
		dstIndex += 4;
		writeInt4( desiredAccess, dst, dstIndex );
		dstIndex += 4;
		writeLong( allocationSize, dst, dstIndex );
		dstIndex += 8;
		writeInt4( extFileAttributes, dst, dstIndex );
		dstIndex += 4;
		writeInt4( shareAccess, dst, dstIndex );
		dstIndex += 4;
		writeInt4( createDisposition, dst, dstIndex );
		dstIndex += 4;
		writeInt4( createOptions, dst, dstIndex );
		dstIndex += 4;
		writeInt4( impersonationLevel, dst, dstIndex );
		dstIndex += 4;
		dst[dstIndex++] = securityFlags;

		return dstIndex - start;
	}
	int writeBytesWireFormat( byte[] dst, int dstIndex ) {
		dst[dstIndex++] = (byte)0x00; // why; SmbComOpenAndX does it too
		return writeString( name, dst, dstIndex ) + 1;
	}
	int readParameterWordsWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readBytesWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	public String toString() {
		return new String( "SmbComNTCreateAndX[" +
			super.toString() +
			",flags=0x" + Log.getHexString( flags, 2 ) +
			",rootDirectoryFid=" + rootDirectoryFid +
			",desiredAccess=0x" + Log.getHexString( desiredAccess, 4 ) +
			",allocationSize=" + allocationSize +
			",extFileAttributes=0x" + Log.getHexString( extFileAttributes, 4 ) +
			",shareAccess=0x" + Log.getHexString( shareAccess, 4 ) +
			",createDisposition=0x" + Log.getHexString( createDisposition, 4 ) +
			",createOptions=0x" + Log.getHexString( createOptions, 8 ) +
			",impersonationLevel=0x" + Log.getHexString( impersonationLevel, 4 ) +
			",securityFlags=0x" + Log.getHexString( securityFlags, 2 ) +
			",name=" + name + "]" );
	}
}
