/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.util.Date;

class SmbComNegotiateResponse extends ServerMessageBlock {

	int dialectIndex,
		securityMode,
		security,
		maxMpxCount,
		maxNumberVcs,
		maxBufferSize,
		maxRawSize,
		sessionKey,
		capabilities,
		serverTimeZone,
		encryptionKeyLength;
	boolean encryptedPasswords,
		signaturesEnabled,
		signaturesRequired;
	long serverTime;
	byte[] encryptionKey;
	String oemDomainName;

	SmbComNegotiateResponse() {
	}

	int writeParameterWordsWireFormat( byte[] dst, int dstIndex ) {
		return 0;
	}
	int writeBytesWireFormat( byte[] dst, int dstIndex ) {
		return 0;
	}
	int readParameterWordsWireFormat( byte[] buffer,
									int bufferIndex ) {
		int start = bufferIndex;
		dialectIndex        = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		if( dialectIndex > 10 ) {
			return bufferIndex - start;
		}
		securityMode        = buffer[bufferIndex++] & 0xFF;
		security            = securityMode & 0x01;
		encryptedPasswords  = ( securityMode & 0x02 ) == 0x02 ? true : false;
		signaturesEnabled   = ( securityMode & 0x04 ) == 0x04 ? true : false;
		signaturesRequired  = ( securityMode & 0x08 ) == 0x08 ? true : false;
		maxMpxCount         = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		maxNumberVcs        = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		maxBufferSize       = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		maxRawSize          = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		sessionKey          = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		capabilities        = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		serverTime          = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		serverTimeZone      = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		encryptionKeyLength = buffer[bufferIndex++] & 0xFF;
		return bufferIndex - start;
	}
	int readBytesWireFormat( byte[] buffer,
									int bufferIndex ) {
		int start = bufferIndex;
		encryptionKey = new byte[encryptionKeyLength];
		System.arraycopy( buffer, bufferIndex,
						encryptionKey, 0, encryptionKeyLength );
		bufferIndex += encryptionKeyLength;
		// at least NT observed reporting oemDomainName in Unicode
		// regardless of flags2 requesting no Unicode in request
		oemDomainName = readString( buffer, bufferIndex, true );
		bufferIndex += stringWireLength( oemDomainName, true );
		return bufferIndex - start;
	}
	public String toString() {
		return new String( "SmbComNegotiateResponse[" +
			super.toString() +
			",wordCount="           + wordCount +
			",dialectIndex="        + dialectIndex +
			",securityMode=0x"      + Log.getHexString( securityMode, 1 ) +
			",security="            + ( security == SECURITY_SHARE ? "share" : "user" ) +
			",encryptedPasswords="  + encryptedPasswords +
			",maxMpxCount="         + maxMpxCount +
			",maxNumberVcs="        + maxNumberVcs +
			",maxBufferSize="       + maxBufferSize +
			",maxRawSize="          + maxRawSize +
			",sessionKey=0x"        + jcifs.util.Log.getHexString( sessionKey, 8 ) +
			",capabilities=0x"      + jcifs.util.Log.getHexString( capabilities, 8 ) +
			",serverTime="          + new Date( serverTime ) +
			",serverTimeZone="      + serverTimeZone +
			",encryptionKeyLength=" + encryptionKeyLength +
			",byteCount="           + byteCount +
			",encryptionKey=0x"     + Log.getHexString( encryptionKey,
												0,
												encryptionKeyLength * 2 ) +
			",oemDomainName="       + oemDomainName + "]" );
	}
}
