/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.util.Date;

class SmbComOpenAndX extends AndXServerMessageBlock {

	// flags (not the same as flags constructor argument)
	static final int FLAGS_RETURN_ADDITIONAL_INFO = 0x01;
	static final int FLAGS_REQUEST_OPLOCK         = 0x02;
	static final int FLAGS_REQUEST_BATCH_OPLOCK   = 0x04;

	// Access Mode Encoding for desiredAccess
	static final int SHARING_COMPATIBILITY           = 0;
	static final int SHARING_DENY_READ_WRITE_EXECUTE = 1;
	static final int SHARING_DENY_WRITE              = 2;
	static final int SHARING_DENY_READ_EXECUTE       = 3;
	static final int SHARING_DENY_NONE               = 4;

	static final int DO_NOT_CACHE  = 0x1000; // bit 12
	static final int WRITE_THROUGH = 0x4000; // bit 14

	static final int REFERENCE_LOCALITY_SEQUENTIAL  = 1;
	static final int REFERENCE_LOCALITY_RANDOM      = 2;

	int flags,
		desiredAccess,
		searchAttributes,
		fileAttributes,
		creationTime,
		openFunction,
		allocationSize;
	String fileName;

	// flags is NOT the same as flags member

	SmbComOpenAndX( String fileName, int flags, ServerMessageBlock andx ) {
		super( andx );
		this.fileName = fileName;
		command = SMB_COM_OPEN_ANDX;

		desiredAccess = ( flags >> 16 ) & 0xFFFF;
		desiredAccess |= ( SHARING_DENY_NONE << 4 ) & 0x70;
		searchAttributes = ATTR_DIRECTORY | ATTR_HIDDEN | ATTR_SYSTEM;
		openFunction = flags & 0xFF;
		if(( openFunction & 0x0F ) != SmbFile.O_TRUNC ) {
			if(( openFunction & SmbFile.O_EXCL ) == 0 ) {
				openFunction |= 0x01; 
			} else {
				openFunction ^= 0x01;
			}
		}
	}

	int writeParameterWordsWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		writeInt2( flags, dst, dstIndex );
		dstIndex += 2;
		writeInt2( desiredAccess, dst, dstIndex );
		dstIndex += 2;
		writeInt2( searchAttributes, dst, dstIndex );
		dstIndex += 2;
		writeInt2( fileAttributes, dst, dstIndex );
		dstIndex += 2;
		creationTime = 0;
		writeInt4( creationTime, dst, dstIndex );
		dstIndex += 4;
		writeInt2( openFunction, dst, dstIndex );
		dstIndex += 2;
		writeInt4( allocationSize, dst, dstIndex );
		dstIndex += 4;
		for( int i = 0; i < 8; i++ ) {
			dst[dstIndex++] = 0x00;
		}

		return dstIndex - start;
	}
	int writeBytesWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		if( useUnicode ) {
			dst[dstIndex++] = (byte)'\0';
		}
		dstIndex += writeString( fileName, dst, dstIndex );

		return dstIndex - start;
	}
	int readParameterWordsWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readBytesWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	public String toString() {
		return new String( "SmbComOpenAndX[" +
			super.toString() +
			",flags=0x" + Log.getHexString( flags, 2 ) +
			",desiredAccess=0x" + Log.getHexString( desiredAccess, 4 ) +
			",searchAttributes=0x" + Log.getHexString( searchAttributes, 4 ) +
			",fileAttributes=0x" + Log.getHexString( fileAttributes, 4 ) +
			",creationTime=" + new Date( creationTime ) +
			",openFunction=0x" + Log.getHexString( openFunction, 2 ) +
			",allocationSize=" + allocationSize +
			",fileName=" + fileName + "]" );
	}
}
