/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

class SmbComSessionSetupAndX extends AndXServerMessageBlock {

	byte[] accountPassword, unicodePassword;
	int passwordLength, unicodePasswordLength;
	int negotiatedMaxBufferSize,
		negotiatedMaxMpxCount,
		vcNumber,
		sessionKey;
	String accountName,
		primaryDomain,
		nativeOs,
		nativeLanMan;

	SmbSession session;

	SmbComSessionSetupAndX( SmbSession session, ServerMessageBlock andx ) {
		super( andx );
		command = SMB_COM_SESSION_SETUP_ANDX;
		this.session = session;
	}

	int writeParameterWordsWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		if( session.transport.server.security == SECURITY_USER &&
													session.password.length() != 0 ) {
			if( session.transport.server.encryptedPasswords ) {
				// encrypted
				accountPassword = SmbSession.getPreNTLMResponse( session.password,
											session.transport.server.encryptionKey );
				unicodePassword = SmbSession.getNTLMResponse( session.password,
											session.transport.server.encryptionKey );
				passwordLength = unicodePasswordLength = 24;
			} else {
				// plain text
				throw new RuntimeException( "plain text passwords not implemented" );
			}
		} else {
			// no password in session setup
			passwordLength = unicodePasswordLength = 0;
		}

		negotiatedMaxBufferSize = session.transport.negotiatedMaxBufferSize;
		negotiatedMaxMpxCount = session.transport.negotiatedMaxMpxCount;
		vcNumber = session.transport.client.vcNumber;
		sessionKey = session.transport.client.sessionKey;

		writeInt2( negotiatedMaxBufferSize, dst, dstIndex );
		dstIndex += 2;
		writeInt2( negotiatedMaxMpxCount, dst, dstIndex );
		dstIndex += 2;
		writeInt2( vcNumber, dst, dstIndex );
		dstIndex += 2;
		writeInt4( sessionKey, dst, dstIndex );
		dstIndex += 4;
		writeInt2( passwordLength, dst, dstIndex );
		dstIndex += 2;
		writeInt2( unicodePasswordLength, dst, dstIndex );
		dstIndex += 2;
		dst[dstIndex++] = (byte)0x00;
		dst[dstIndex++] = (byte)0x00;
		dst[dstIndex++] = (byte)0x00;
		dst[dstIndex++] = (byte)0x00;
		writeInt4( session.transport.client.capabilities, dst, dstIndex );
		dstIndex += 4;

		return dstIndex - start;
	}
	int writeBytesWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		accountName = session.username;
		primaryDomain = session.domain;
		nativeOs = session.transport.client.nativeOs;
		nativeLanMan = session.transport.client.nativeLanMan;

		if( session.transport.server.security == SECURITY_USER &&
													session.password.length() != 0 ) {
			System.arraycopy( accountPassword, 0, dst, dstIndex, passwordLength );
			dstIndex += passwordLength;
			System.arraycopy( unicodePassword, 0, dst, dstIndex, unicodePasswordLength );
			dstIndex += unicodePasswordLength;
		}
		if( useUnicode ) {
			// at least NT 4 observed needing this only with unicode
			dst[dstIndex++] = (byte)'\0';
		}

		dstIndex += writeString( accountName, dst, dstIndex );
		dstIndex += writeString( primaryDomain, dst, dstIndex );
		dstIndex += writeString( nativeOs, dst, dstIndex );
		// at least NT 4 observed with 2 zero bytes here
		dst[dstIndex++] = (byte)0x00;
		dst[dstIndex++] = (byte)0x00;
		dstIndex += writeString( nativeLanMan, dst, dstIndex );

		return dstIndex - start;
	}
	int readParameterWordsWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readBytesWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	public String toString() {
		String result = new String( "SmbComSessionSetupAndX[" +
			super.toString() +
			",maxBufferSize=" + negotiatedMaxBufferSize +
			",maxMpxCount=" + negotiatedMaxMpxCount +
			",vcNumber=" + vcNumber +
			",sessionKey=" + sessionKey +
			",passwordLength=" + passwordLength +
			",unicodePasswordLength=" + unicodePasswordLength +
			",capabilities=" + session.transport.client.capabilities +
			",accountPassword=" + accountPassword +
			",accountName=" + accountName +
			",primaryDomain=" + primaryDomain +
			",nativeOs=" + nativeOs +
			",nativeLanMan=" + nativeLanMan + "]" );
		return result;
	}
}
