/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

class SmbComTreeConnectAndX extends AndXServerMessageBlock {

	SmbSession session;
	boolean disconnectTid = false;
	String path, service;
	byte[] password;
	int passwordLength;

	SmbComTreeConnectAndX( SmbSession session, String path,
								String service, ServerMessageBlock andx ) {

		super( andx );
		this.session = session;
		this.path = path;
		this.service = service;
		command = SMB_COM_TREE_CONNECT_ANDX;
	}

	int writeParameterWordsWireFormat( byte[] dst, int dstIndex ) {

		if( session.transport.server.security == SECURITY_SHARE &&
													session.password.length() != 0 ) {
			if( session.transport.server.encryptedPasswords ) {
				// encrypted
				password = SmbSession.getPreNTLMResponse( session.password,
											session.transport.server.encryptionKey );
				passwordLength = 24;
			} else {
				// plain text
				throw new RuntimeException( "plain text passwords not implemented" );
			}
		} else {
			// no password in tree connect
			passwordLength = 1;
		}

		dst[dstIndex++] = (byte)0x00;
		dst[dstIndex++] = disconnectTid ? (byte)0x01 : (byte)0x00;
		writeInt2( passwordLength, dst, dstIndex );
		return 4;
	}
	int writeBytesWireFormat( byte[] dst, int dstIndex ) {
		int start = dstIndex;

		if( session.transport.server.security == SECURITY_SHARE &&
													session.password.length() != 0 ) {
			if( session.transport.server.encryptedPasswords ) {
				// encrypted
				System.arraycopy( password, 0, dst, dstIndex, passwordLength );
				dstIndex += passwordLength;
			} else {
				// plain text
				throw new RuntimeException( "plain text passwords not implemented" );
			}
		} else {
			// no password in tree connect
			dst[dstIndex++] = (byte)0x00;
		}
		dstIndex += writeString( path, dst, dstIndex );
		System.arraycopy( service.getBytes(), 0, dst, dstIndex, service.length() );
		dstIndex += service.length();
		dst[dstIndex++] = (byte)'\0';

		return dstIndex - start;
	}
	int readParameterWordsWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readBytesWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	public String toString() {
		String result = new String( "SmbComTreeConnectAndX[" +
			super.toString() +
			",disconnectTid=" + disconnectTid +
			",passwordLength=" + passwordLength +
			",password=" + Log.getHexString(
						password, passwordLength, 0 ) +
			",path=" + path +
			",service=" + service + "]" );
		return result;
	}
}

