/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.io.IOException;

class SmbException extends IOException {

	// error classes
	static final int SUCCESS = 0;
	static final int ERRDOS = 0x01;
	static final int ERRSRV = 0x02;
	static final int ERRHRD = 0x03;
	static final int ERRCMD = 0xFF;

	// dos error codes
	static final int ERRbadfunc   = 1;
	static final int ERRbadfile   = 2;
	static final int ERRbadpath   = 3;
	static final int ERRnoaccess  = 5;
	static final int ERRbadfid    = 6;
	static final int ERRbadparm   = 87;
	static final int ERRfilexists = 80;
	static final int ERRinvname   = 123;
	// srv error codes
	static final int ERRerror = 1;
	static final int ERRbadpw = 2;
	static final int ERRaccess = 4;
	static final int ERRinvnid = 5;
	static final int ERRinvnetname = 6;
	static final int ERRbaduid = 91;
	// hrd error codes
	static final int ERRnowrite = 19;

	int errorClass;
	int errorCode;

	static String getErrorString( int errorClass, int errorCode ) {
		String result = "";
		switch( errorClass ) {
			case SUCCESS:
				result += "SUCCESS";
				break;
			case ERRDOS:
				result += "ERRDOS/";
				switch( errorCode ) {
					case ERRbadfunc:
						result += "ERRbadfunc: Invalid function";
						break;
					case ERRbadfile:
						result += "ERRbadfile: File not found";
						break;
					case ERRbadpath:
						result += "ERRbadpath: Directory invalid";
						break;
					case ERRnoaccess:
						result += "ERRnoaccess: Access denied";
						break;
					case ERRbadparm:
						result += "ERRbadparm: Invalid parameter";
						break;
					case ERRinvname:
						result += "ERRinvname: Invalid name";
						break;
					case ERRfilexists:
						result += "ERRfilexists: File exists";
						break;
					case ERRbadfid:
						result += "ERRbadfid: Invalid file handle";
						break;
					default:
						result += "Ehh, one of the other ones. Please update " +
										"getErrorString for errorCode=" + errorCode;
				}
				break;
			case ERRSRV:
				result += "ERRSRV/";
				switch( errorCode ) {
					case ERRerror:
						result += "ERRerror: Non-specific error code";
						break;
					case ERRbadpw:
						result += "ERRbadpw: Bad password";
						break;
					case ERRinvnid:
						result += "ERRinvnid: The Tid specified was invalid";
						break;
					case ERRinvnetname:
						result += "ERRinvnetname: Invalid network name in tree connect, " + 
										"service not found";
						break;
					case ERRbaduid:
						result += "ERRbaduid: The UID is not known as a valid user " +
										"identifier on this session";
						break;
					case ERRaccess:
						result += "ERRaccess: The client does not have the necessary " +
										"access rights for the requested function";
						break;
					default:
						result += "Ehh, one of the other ones. Please update " +
								"getErrorString for errorCode=" + errorCode;
				}
				break;
			case ERRHRD:
				result += "ERRHRD/";
				switch( errorCode ) {
					case ERRnowrite:
						result += "Attempt to write on write-protected media";
						break;
					default:
						result += "Ehh, one of the other ones. Please update " +
								"getErrorString for errorCode=" + errorCode;
				}
				break;
			case ERRCMD:
				result += "ERRCMD: Command was not in the \"SMB\" format";
				break;
			default:
				result += "unknown error class: " + errorClass;
		}
		return result;
	}
	static String getErrorString( int code ) {
		if( code == 0 ) {
			return "0x00000000";
		}
		return getErrorString( code & 0xFF, ( code >> 16 ) & 0xFFFF );
	}

	public SmbException() {
	}
	public SmbException( int code ) {
		super( getErrorString( code & 0xFF, ( code >> 16 ) & 0xFFFF ));
		errorClass = code & 0xFF;
		errorCode = ( code >> 16 ) & 0xFFFF;
	}
}
