/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.ListIterator;

final class SmbURL {
	String host;
	String domain;
	String username;
	String password;
	int port;
	String share;
	String canonicalPath;
	String path;
	String file;

	/*
	 * smb:// [[ domain; ] user [ :pass ] @ ] host [ :port ] /share [ /dir|file ]
	 *
	 * smb://hendrix/download/
	 * smb://nycdom;mike@angus/tmp/classes/Main.class
	 * smb://mike:sw@#ii@angus/tmp/classes
	 * smb://nycdom;mike:sw@#ii@hendrix/download/docs/index.html
	 */

	SmbURL( String url ) throws MalformedURLException {
		this( url, null, 4, url.length() );
	}
	SmbURL( String url, String name ) throws MalformedURLException {
		this( url, name, 4, url.length() );
	}
	SmbURL( String url, String name, int start, int limit ) throws MalformedURLException {
		try {
			if( name != null && name.startsWith( "smb://" )) {
				url = name;
				name = null;
				start = 4;
				limit = url.length();
			}
			int s = url.indexOf( '/', start + 2 );
			if( s == -1 ) {
				s = limit;
			}
			int a = url.lastIndexOf( '@', s );
			int p = url.lastIndexOf( ':', s );
			int h = Math.max( a + 1, start + 2 );
	
			if( p > h ) {
				port = Integer.parseInt( url.substring( p + 1, s ));
			}
			host = url.substring( h, ( p > h ? p : s ));
	
			if( a != -1 ) {
				int c = url.indexOf( ':', start + 3 );
				if( c != -1 ) {
					password = url.substring( c + 1, a );
					a = c;
				}
				int b = url.indexOf( ';', start + 3 );
				if( b != -1 && b < a ) {
					domain = url.substring( start + 2, b );
				} else {
					b = start + 1;
				}
				if( a > b ) {
					username = url.substring( b + 1, a );
				}
			}
	
			if( name == null || name.length() == 0 ) {
				name = url;
			} else if( name.charAt( 0 ) == '/' ) {
				s = 1;
				limit = name.length();
			} else {
				name = url.substring( s ) + "/" + name;
				s = 1;
				limit = name.length();
			}
	
			ArrayList stk = new ArrayList();
			StringBuffer sb = new StringBuffer();
			String str;
			int n = s;
			int sp = 0;
	
			while( n < limit ) {
				if( name.charAt( n ) == '/' ) {
					if( n > s ) {
						str = name.substring( s, n );
						sb.append( '/' ).append( str );
						if( str.equals( ".." ) && sp != 0 ) {
							stk.remove( --sp );
						} else if( str.equals( "." )) {
						} else {
							stk.add( str );
							sp++;
						}
					}
					s = n + 1;
				}
				n++;
			}
			if( n > s ) {
				str = name.substring( s, n );
				sb.append( '/' ).append( str );
				if( str.equals( ".." ) && sp != 0 ) {
					stk.remove( --sp );
				} else if( str.equals( "." )) {
				} else {
					stk.add( str );
					sp++;
				}
			}
			path = sb.toString();
			file = path.substring( path.lastIndexOf( '/' ) + 1, path.length() );
	
			share = (String)stk.get( 0 );
			ListIterator li = stk.listIterator( 1 );
			sb.setLength( 0 );
			while( li.hasNext() ) {
				sb.append( '/' ).append( li.next() );
			}
			if( sb.length() > 0 ) {
				canonicalPath = sb.toString();
			} else {
				canonicalPath = "";
			}
		} catch( Exception e ) {
			throw new MalformedURLException( url + ", " + name );
		}
	}

	public boolean equals( Object obj ) {
		return obj instanceof SmbURL && obj.hashCode() == hashCode();
	}
	public int hashCode() {
		return toString().hashCode();
	}
	String getCanonicalPath() {
		StringBuffer sb = new StringBuffer( "smb://" );
		if( username != null ) {
			if( domain != null ) {
				sb.append( domain ).append( ';' );
			}
			sb.append( username );
			if( password != null ) {
				sb.append( ':' ).append( password );
			}
			sb.append( '@' );
		}
		sb.append( host );
		// should probably not put the netbios session service
		// port in here like this :~(
		if( port != 0 && port != 139 ) {
			sb.append( ":" ).append( port );
		}
		sb.append( '/' ).append( share ).append( canonicalPath );
		return sb.toString();
	}
	public String toString() {
		StringBuffer sb = new StringBuffer( "smb://" );
		if( username != null ) {
			if( domain != null ) {
				sb.append( domain ).append( ';' );
			}
			sb.append( username );
			if( password != null ) {
				sb.append( ':' ).append( password );
			}
			sb.append( '@' );
		}
		sb.append( host );
		// should probably not put the netbios session service
		// port in here like this :~(
		if( port != 0 && port != 139 ) {
			sb.append( ":" ).append( port );
		}
		sb.append( path );
		return sb.toString();
	}
}

