/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.net.URLConnection;
import java.net.URL;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/** 
 * This class represents a communications link between an application and
 * an SMB resource represented by a {@link java.net.URL}. This class is
 * returned by the java.net.URL openConnection method when an smb URL is
 * used. See the {@link jcifs.smb.SmbFile} class for a detailed decription of
 * the smb URL syntax with examples.
 *
 * In general this class is deficient. Adding more functionality is
 * strait forward. If you require more functionality please express your
 * intrest to the jcifs@samba.org mailing list.
 */ 

public class SmbURLConnection extends URLConnection {

	SmbURL url;
	SmbFile file;

	SmbURLConnection( URL u, SmbURL url ) {
		super( u );
		this.url = url;
	}

/** 
 * Issue the actual socket IO necesary to open a connection with the server
 * hosting the target resource.
 */ 

	public void connect() throws IOException {
		if( connected ) {
			return;
		}
		file = new SmbFile( url );
	}

/**
 * Return the length in bytes of the resource specified by the URL.
 */

	public int getContentLength() {
		try {
			connect();
			return file.exists() ? file.size : 0;
		} catch( IOException ioe ) {
		}
		return 0;
	}

/**
 * Retrieves the last modified time of the file or directory specified by
 * the URL for this connection.
 */

	public long getDate() {
		return getLastModified();
	}

/**
 * Retrieves the last modified time of the file or directory specified by
 * the URL for this connection.
 */

	public long getLastModified() {
		try {
			connect();
			return file.lastModified();
		} catch( IOException ioe ) {
		}
		return 0L;
	}

/**
 * Create an <code>InputStream</code> from which bytes may be read from
 * the taget specified by the URL of this connection.
 */

	public InputStream getInputStream() throws IOException {
		connect();
		return new SmbFileInputStream( file );
	}

/**
 * Return a String representing this object.
 */

	public String toString() {
		return new String( "SmbURLConnection[url=" + url.toString() + "]" );
	}
}
