/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import jcifs.util.Config;

class Trans2FindFirst2 extends SmbComTransaction {


	// information levels

	static final int SMB_INFO_STANDARD                 = 1;
	static final int SMB_INFO_QUERY_EA_SIZE            = 2;
	static final int SMB_INFO_QUERY_EAS_FROM_LIST      = 3;
	static final int SMB_FIND_FILE_DIRECTORY_INFO      = 0x101;
	static final int SMB_FIND_FILE_FULL_DIRECTORY_INFO = 0x102;
	static final int SMB_FILE_NAMES_INFO               = 0x103;
	static final int SMB_FILE_BOTH_DIRECTORY_INFO      = 0x104;

	// flags

	static final int FLAGS_CLOSE_AFTER_THIS_REQUEST = 0x01;
	static final int FLAGS_CLOSE_IF_END_REACHED     = 0x02;
	static final int FLAGS_RETURN_RESUME_KEYS       = 0x04;
	static final int FLAGS_RESUME_FROM_PREVIOUS_END = 0x08;
	static final int FLAGS_FIND_WITH_BACKUP_INTENT  = 0x10;

	
	static final int DEFAULT_MAX_DATA_COUNT = 800;

	int searchAttributes;
	int searchCount;
	int flags;
	int informationLevel;
	int searchStorageType = 0;
	String filename;

	Trans2FindFirst2( String filename ) {
		this.filename = filename;
		subCommand = TRANS2_FIND_FIRST2;

		searchAttributes = ATTR_DIRECTORY | ATTR_HIDDEN | ATTR_SYSTEM;
		searchCount = 6;
		flags = 0x00;
		informationLevel = SMB_FILE_BOTH_DIRECTORY_INFO;

		totalDataCount = 0;
		maxParameterCount = 10;
		maxDataCount = Config.getInt( "jcifs.smb.client.listSize",
													DEFAULT_MAX_DATA_COUNT );
		maxSetupCount = 0;
	}

	public Object nextElement() {
		totalParameterCount = 12 + stringWireLength( filename );
		return super.nextElement();
	}
	int writeParametersWireFormat( byte[] dst, int dstIndex, int len ) {
		int start = dstIndex;

		writeInt2( searchAttributes, dst, dstIndex );
		dstIndex += 2;
		writeInt2( searchCount, dst, dstIndex );
		dstIndex += 2;
		writeInt2( flags, dst, dstIndex );
		dstIndex += 2;
		writeInt2( informationLevel, dst, dstIndex );
		dstIndex += 2;
		writeInt4( searchStorageType, dst, dstIndex );
		dstIndex += 4;
		dstIndex += writeString( filename, dst, dstIndex );

		return dstIndex - start;
	}
	int writeDataWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int readParametersWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readDataWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	public String toString() {
		return new String( "Trans2FindFirst2[" + super.toString() +
			",searchAttributes=0x" + Log.getHexString( searchAttributes, 2 ) +
			",searchCount=" + searchCount +
			",flags=0x" + Log.getHexString( flags, 2 ) +
			",informationLevel=0x" + Log.getHexString( informationLevel, 3 ) +
			",searchStorageType=" + searchStorageType +
			",filename=" + filename + "]" );
	}
}
