/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.io.UnsupportedEncodingException;
import java.util.Date;

class Trans2FindFirst2Response extends SmbComTransactionResponse {

	// information levels

	static final int SMB_INFO_STANDARD                 = 1;
	static final int SMB_INFO_QUERY_EA_SIZE            = 2;
	static final int SMB_INFO_QUERY_EAS_FROM_LIST      = 3;
	static final int SMB_FIND_FILE_DIRECTORY_INFO      = 0x101;
	static final int SMB_FIND_FILE_FULL_DIRECTORY_INFO = 0x102;
	static final int SMB_FILE_NAMES_INFO               = 0x103;
	static final int SMB_FILE_BOTH_DIRECTORY_INFO      = 0x104;

	class SmbFindFileBothDirectoryInfo {
		int nextEntryOffset;
		int fileIndex;
		long creationTime;
		long lastAccessTime;
		long lastWriteTime;
		long changeTime;
		long endOfFile;
		long allocationSize;
		int extFileAttributes;
		int fileNameLength;
		int eaSize;
		int shortNameLength;
		String shortName;
		String filename;

		public String toString() {
			return new String( "SmbFindFileBothDirectoryInfo[" +
				"nextEntryOffset=" + nextEntryOffset +
				",fileIndex=" + fileIndex +
				",creationTime=" + new Date( creationTime ) +
				",lastAccessTime=" + new Date( lastAccessTime ) +
				",lastWriteTime=" + new Date( lastWriteTime ) +
				",changeTime=" + new Date( changeTime ) +
				",endOfFile=" + endOfFile +
				",allocationSize=" + allocationSize +
				",extFileAttributes=" + extFileAttributes +
				",fileNameLength=" + fileNameLength +
				",eaSize=" + eaSize +
				",shortNameLength=" + shortNameLength +
				",shortName=" + shortName +
				",filename=" + filename + "]" );
		}
	}

	int sid;
	int searchCount;
	boolean isEndOfSearch;
	int eaErrorOffset;
	int lastNameOffset;
	String lastName;

	SmbFindFileBothDirectoryInfo[] results;

	Trans2FindFirst2Response() {
		subCommand = SmbComTransaction.TRANS2_FIND_FIRST2;
	}

	String readString( byte[] src, int srcIndex, int len ) {
		String str = null;
		if( useUnicode ) {
			try {
				str = new String( src, srcIndex, len, "UnicodeLittle" );
			} catch( UnsupportedEncodingException uee ) {
				Log.printStackTrace( "smb exception", uee );
			}
		} else {
			str = new String( src, srcIndex, len );
		}
		return str;
	}
	int writeParametersWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int writeDataWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int readParametersWireFormat( byte[] buffer, int bufferIndex ) {
		int start = bufferIndex;

		if( subCommand == SmbComTransaction.TRANS2_FIND_FIRST2 ) {
			sid = readInt2( buffer, bufferIndex );
			bufferIndex += 2;
		}
		searchCount = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		isEndOfSearch = ( buffer[bufferIndex] & 0x01 ) == 0x01 ? true : false;
		bufferIndex += 2;
		eaErrorOffset = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		lastNameOffset = readInt2( buffer, bufferIndex );
		bufferIndex += 2;

		return bufferIndex - start;
	}
	int readDataWireFormat( byte[] buffer, int bufferIndex ) {
		int start = bufferIndex;

		results = new SmbFindFileBothDirectoryInfo[searchCount];

		for( int i = 0; i < searchCount; i++ ) {
			results[i] = new SmbFindFileBothDirectoryInfo();

			results[i].nextEntryOffset = readInt4( buffer, bufferIndex );
			results[i].fileIndex = readInt4( buffer, bufferIndex + 4 );
			results[i].creationTime = readTime( buffer, bufferIndex + 8 );
			results[i].lastAccessTime = readTime( buffer, bufferIndex + 16 );
			results[i].lastWriteTime = readTime( buffer, bufferIndex + 24 );
			results[i].changeTime = readTime( buffer, bufferIndex + 32 );
			results[i].endOfFile = readLong( buffer, bufferIndex + 40 );
			results[i].allocationSize = readLong( buffer, bufferIndex + 48 );
			results[i].extFileAttributes = readInt4( buffer, bufferIndex + 56 );
			results[i].fileNameLength = readInt4( buffer, bufferIndex + 60 );
			results[i].eaSize = readInt4( buffer, bufferIndex + 64 );
			results[i].shortNameLength = buffer[bufferIndex + 68] & 0xFF;
			results[i].shortName = new String( buffer, bufferIndex + 70,
														results[i].shortNameLength );
			results[i].filename = readString( buffer, bufferIndex + 94,
														results[i].fileNameLength );
			int off = bufferIndex - dataOffset;
			if( lastNameOffset >= off &&
						lastNameOffset <= ( off + results[i].nextEntryOffset )) {
				lastName = results[i].filename;
			}
//Log.println( Log.DEBUGGING, "info entry", results[i].toString() );

			bufferIndex += results[i].nextEntryOffset;
		}

		/* last nextEntryOffset for NT 4(but not 98) is 0 so we must
		 * use dataCount or our accounting will report an error for NT :~(
		 */
		
		//return bufferIndex - start;

		return dataCount;
	}
	public String toString() {
		String c;
		if( subCommand == SmbComTransaction.TRANS2_FIND_FIRST2 ) {
			c = "Trans2FindFirst2Response[";
		} else {
			c = "Trans2FindNext2Response[";
		}
		return new String( c + super.toString() +
			",sid=" + sid +
			",searchCount=" + searchCount +
			",isEndOfSearch=" + isEndOfSearch +
			",eaErrorOffset=" + eaErrorOffset +
			",lastNameOffset=" + lastNameOffset +
			",lastName=" + lastName + "]" );
	}
}
