/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.io.UnsupportedEncodingException;

class Trans2QueryFSInformationResponse extends SmbComTransactionResponse {

	// information levels
	static final int SMB_QUERY_FS_ATTRIBUTE_INFO = 0x105;

	class SmbQueryFSAttributeInfo implements Info {
		int attributes;
		int maxNameLength;
		String name;

		public int getAttributes() {
			return attributes;
		}
		public long getLastWriteTime() {
			return 0;
		}
		public int getSize() {
			return 0;
		}
		public String toString() {
			return new String( "SmbQueryFSAttributeInfo[" +
				"attributes=0x" + Log.getHexString( attributes, 4 ) +
				",maxNameLength=" + maxNameLength +
				",name=" + name + "]" );
		}
	}

	Info info;

	Trans2QueryFSInformationResponse() {
		subCommand = SmbComTransaction.TRANS2_QUERY_FS_INFORMATION;
	}

	int writeParametersWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int writeDataWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int readParametersWireFormat( byte[] buffer, int bufferIndex ) {
		return 0;
	}
	int readDataWireFormat( byte[] buffer, int bufferIndex ) {
		int start = bufferIndex;

		SmbQueryFSAttributeInfo info = new SmbQueryFSAttributeInfo();
		info.attributes = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.maxNameLength = readInt2( buffer, bufferIndex );
		bufferIndex += 4;
		int nameLength = readInt2( buffer, bufferIndex );
		bufferIndex += 4;
		try {
			info.name = new String( buffer, bufferIndex, nameLength, "UnicodeLittle" );
		} catch( UnsupportedEncodingException uee ) {
			Log.printStackTrace( "", uee );
		}
		bufferIndex += nameLength;
		this.info = info;

		return bufferIndex - start;
	}
	public String toString() {
		return new String( "Trans2QueryFSInformationResponse[" +
			super.toString() + "]" );
	}
}
