/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <mballen@erols.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.smb;

import java.util.Date;

class Trans2QueryPathInformationResponse extends SmbComTransactionResponse {

	// information levels
	static final int SMB_INFO_IS_NAME_VALID    = 6;
	static final int SMB_QUERY_FILE_BASIC_INFO = 0x101;
	static final int SMB_QUERY_FILE_ALL_INFO   = 0x107;

	class SmbQueryFileBasicInfo implements Info {
		long creationTime;
		long lastAccessTime;
		long lastWriteTime;
		long changeTime;
		int attributes;

		public int getAttributes() {
			return attributes;
		}
		public long getLastWriteTime() {
			return lastWriteTime;
		}
		public int getSize() {
			return 0;
		}
		public String toString() {
			return new String( "SmbQueryFileBasicInfo[" +
				"creationTime=" + new Date( creationTime ) +
				",lastAccessTime=" + new Date( lastAccessTime ) +
				",lastWriteTime=" + new Date( lastWriteTime ) +
				",changeTime=" + new Date( changeTime ) +
				",attributes=0x" + Log.getHexString( attributes, 4 ) + "]" );
		}
	}
	class SmbQueryFileAllInfo implements Info {
		long creationTime;
		long lastAccessTime;
		long lastWriteTime;
		long changeTime;
		int attributes;
		long allocationSize;
		long endOfFile;
		int numberOfLinks;
		boolean deletePending;
		boolean directory;
		long indexNumber;
		int eaSize;
		int accessFlags;
		long currentByteOffset;
		int mode;
		int alignmentRequired;
		int fileNameLength;
		String fileName;

		public int getAttributes() {
			return attributes;
		}
		public long getLastWriteTime() {
			return lastWriteTime;
		}
		public int getSize() {
			return (int)allocationSize;
		}
		public String toString() {
			return new String( "SmbQueryFileAllInfo[" +
				",creationTime=" + new Date( creationTime ) +
				",lastAccessTime=" + new Date( lastAccessTime ) +
				",lastWriteTime=" + new Date( lastWriteTime ) +
				",changeTime=" + new Date( changeTime ) +
				",attributes=0x" + Log.getHexString( attributes, 4 ) +
				",allocationSize=" + allocationSize +
				",endOfFile=" + endOfFile +
				",numberOfLinks=" + numberOfLinks +
				",deletePending=" + deletePending +
				",directory=" + directory +
				",indexNumber=" + indexNumber +
				",eaSize=" + eaSize +
				",accessFlags=0x" + Log.getHexString( accessFlags, 8 ) +
				",currentByteOffset=" + currentByteOffset +
				",mode=0x" + Log.getHexString( mode, 8 ) +
				",alignmentRequired=0x" + Log.getHexString( alignmentRequired, 8 ) +
				",fileNameLength=" + fileNameLength +
				",fileName=" + fileName + "]" );
		}
	}

	int informationLevel;
	Info info;

	Trans2QueryPathInformationResponse( int informationLevel ) {
		this.informationLevel = informationLevel;
		subCommand = SmbComTransaction.TRANS2_QUERY_PATH_INFORMATION;
	}

	int writeParametersWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int writeDataWireFormat( byte[] dst, int dstIndex, int len ) {
		return 0;
	}
	int readParametersWireFormat( byte[] buffer, int bufferIndex ) {
		// observed two zero bytes here with at least win98
		return 2;
	}
	int readDataWireFormat( byte[] buffer, int bufferIndex ) {
		switch( informationLevel ) {
			case SMB_INFO_IS_NAME_VALID:
				return 0;
			case SMB_QUERY_FILE_BASIC_INFO:
				return readSmbQueryFileBasicInfoWireFormat( buffer, bufferIndex );
			case SMB_QUERY_FILE_ALL_INFO:
				return readSmbQueryFileAllInfoWireFormat( buffer, bufferIndex );
			default:
				return 0;
		}
	}
	int readSmbQueryFileBasicInfoWireFormat( byte[] buffer, int bufferIndex ) {
		int start = bufferIndex;

		SmbQueryFileBasicInfo info = new SmbQueryFileBasicInfo();
		info.creationTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.lastAccessTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.lastWriteTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.changeTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.attributes = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		this.info = info;

Log.println( Log.DEBUGGING, "Trans2QueryPathInformationResponse result", info );

		return bufferIndex - start;
	}
	int readSmbQueryFileAllInfoWireFormat( byte[] buffer, int bufferIndex ) {
		int start = bufferIndex;

		SmbQueryFileAllInfo info = new SmbQueryFileAllInfo();
		info.creationTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.lastAccessTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.lastWriteTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.changeTime = readTime( buffer, bufferIndex );
		bufferIndex += 8;
		info.attributes = readInt2( buffer, bufferIndex );
		bufferIndex += 2;
		info.allocationSize = readLong( buffer, bufferIndex );
		bufferIndex += 8;
		info.endOfFile = readLong( buffer, bufferIndex );
		bufferIndex += 8;
		info.numberOfLinks = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.deletePending = ( buffer[bufferIndex++] & 0xFF ) > 0;
		info.directory = ( buffer[bufferIndex++] & 0xFF ) > 0;
		info.indexNumber = readLong( buffer, bufferIndex );
		bufferIndex += 8;
		info.eaSize = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.accessFlags = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.currentByteOffset = readLong( buffer, bufferIndex );
		bufferIndex += 8;
		info.mode = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.alignmentRequired = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.fileNameLength = readInt4( buffer, bufferIndex );
		bufferIndex += 4;
		info.fileName = new String( buffer, bufferIndex, info.fileNameLength );
		bufferIndex += info.fileNameLength;
		this.info = info;

		return bufferIndex - start;
	}
	public String toString() {
		return new String( "Trans2QueryPathInformationResponse[" +
			super.toString() + "]" );
	}
}
